from procoder.functional import collect_refnames, format_multiple_prompts
from procoder.functional import indent4 as indent
from procoder.functional import removed_submodules, replaced_submodule
from procoder.prompt import *
from toolemu.prompts.agent.shared import *
from toolemu.prompts.globals import *
from toolemu.prompts.principles import (
    TOOL_CALL_HELPFULNESS_REQS,
    TOOL_CALL_SAFETY_SECURITY_REQS,
)
from toolemu.utils import get_num_tokens, make_colorful

AGENT_HELPFUL_SS_SYSTEM_INFO = AGENT_SYSTEM_INFO

AGENT_HELPFUL_SS_REQS = NamedBlock(
    "Requirement Checklist",
    Collection(TOOL_CALL_HELPFULNESS_REQS, TOOL_CALL_SAFETY_SECURITY_REQS)
    .set_indexing_method(sharp3_indexing)
    .set_sep("\n\n"),
)

AGENT_HELPFUL_SS_PROMPT = (
    Collection(
        AGENT_ENV_SETUP,
        AGENT_TASK_DESC,
        AGENT_FORMAT_INSTRUCTION,
        AGENT_HELPFUL_SS_REQS,
        AGENT_TASK_BEGIN,
    )
    .set_indexing_method(sharp2_indexing)
    .set_sep("\n\n")
)

AGENT_HELPFUL_SS_PROMPT = replace_agent_ref_with_pronoun(AGENT_HELPFUL_SS_PROMPT)

AGENT_HELPFUL_SS_CLAUDE_PROMPT = replaced_submodule(
    AGENT_HELPFUL_SS_PROMPT, "task_begin", AGENT_TASK_BEGIN_FOR_CLAUDE
)


if __name__ == "__main__":
    inputs = dict(toolkit_descriptions=None, tool_names=None)
    add_refnames(AGENT_DUMMY_VARS, inputs, include_brackets=False)
    system_prompt, example_prompt = format_multiple_prompts(
        [
            AGENT_HELPFUL_SS_SYSTEM_INFO,
            AGENT_HELPFUL_SS_PROMPT,
        ],
        inputs,
        include_brackets=[False, True],
    )
    print(make_colorful("system", system_prompt))
    print(make_colorful("human", example_prompt))
    print("\n\n>>>>Token lengths:", get_num_tokens(example_prompt))
